/*
 * Copyright (C) 2013  Lucas Skldqvist <frusen@gungre.ch>
 *
 * This file is part of Dragbuilder. 
 *
 * Dragbuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dragbuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dragbuilder.  If not, see <http://www.gnu.org/licenses/>. 
 */

#include <errno.h>
#include <FL/fl_ask.H>
#include <FL/Fl_Native_File_Chooser.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Return_Button.H>

#include "terminal.h"
#include "editorwindow.h"

void save_cb();
void saveas_cb();

int
check_save (void)
{
  if (!changed) return 1;

  int r = fl_choice ("The current file has not been saved.\n"
                     "Would you like to save it now?",
                     "Cancel", "Save", "Don't Save");

  if (r == 1)
    {
      save_cb(); // Save the file...
      return !changed;
    }

  return (r == 2) ? 1 : 0;
}

void
load_file (const char *newfile, int ipos)
{
  loading = 1;
  int insert = (ipos != -1);
  changed = insert;
  if (!insert) filename.assign ("");
  int r;
  if (!insert) r = textbuf->loadfile(newfile);
  else r = textbuf->insertfile(newfile, ipos);
  changed = changed || textbuf->input_file_was_transcoded;
  if (r)
    fl_alert("Error reading from file \'%s\':\n%s.", newfile, strerror(errno));
  else
    if (!insert) filename.assign (newfile);
  loading = 0;
  textbuf->call_modify_callbacks();
}

void save_file(std::string newfile) {
  if (textbuf->savefile(newfile.c_str ()))
    fl_alert("Error writing to file \'%s\':\n%s.", newfile.c_str (), strerror(errno));
  else
    filename.assign (newfile);
  changed = 0;
  textbuf->call_modify_callbacks();
}

void
about_cb (Fl_Widget*, void*)
{
  fl_message_title ("About");
  fl_message ("Dragbuilder - A fast and light Dragscript builder.\nVersion 0.1");
}

void
apply_cb (Fl_Widget*, void *v)
{
  EditorWindow* w = (EditorWindow*)v;
  int pos;
  int temp;

  /* Find the (C) character. */
  if (textbuf->search_forward (0, "(C)", &pos) == 1)
    {
      /* Move ahead four positions so we are in front of the years. */
      pos += 4;
      temp = pos;

      /* Find the two spaces before the author's name. */
      if (textbuf->search_forward (temp, "  ", &pos) == 1)
        {
          /* Replace it with what the user has selected. */
          textbuf->replace (temp, pos, w->input_field[IP_YEARS]->value ());
        }
    }

  /* Find the two spaces before the author's name. */
  if (textbuf->search_forward (0, "  ", &pos) == 1)
    {
      /* Move ahead two positions so we are just in front of the author's
         name. */
      pos += 2;
      temp = pos;

      /* Now look for the " <" string just after the author's name and replace
         it. */
      if (textbuf->search_forward (temp, " <", &pos) == 1)
        {
          textbuf->replace (temp, pos, w->input_field[IP_AUTHOR]->value ());
        }
    }

  /* Look for the " <" string just after the author's name. */
  if (textbuf->search_forward (0, " <", &pos) == 1)
    {
      /* Move ahead two positions so we are just in front of the e-mail
         address. */
      pos += 2;
      temp = pos;

      /* Look for the ">" at the end of the e-mail address. */
      if (textbuf->search_forward (temp, ">", &pos) == 1)
        {
          textbuf->replace (temp, pos, w->input_field[IP_EMAIL]->value ());
        }
    }

  /* Look for "# Basic". */
  if (textbuf->search_forward (0, "# Basic", &pos) == 1)
    {
      /* Move down one line and go forward two characters. */
      pos = textbuf->skip_lines (pos, 1);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_PACKAGE]->value ());

      /* Move down one line and go forward two characters. */
      pos = textbuf->skip_lines (pos, 1);
      pos = textbuf->line_start (pos);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_VERSION]->value ());

      /* Move down two lines and go forward two characters. */
      pos = textbuf->skip_lines (pos, 2);
      pos = textbuf->line_start (pos);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_BUILD]->value ());
    }
}

// TODO: check how many authors there are
void
extract_cb (Fl_Widget*, void *v)
{
  EditorWindow* w = (EditorWindow*)v;
  int pos;
  int temp;

  /* Find the (C) character. */
  if (textbuf->search_forward (0, "(C)", &pos) == 1)
    {
      /* Move ahead four positions so we are in front of the years. */
      pos += 4;
      temp = pos;

      /* Find the two spaces before the author's name. */
      if (textbuf->search_forward (temp, "  ", &pos) == 1)
        {
          textbuf->select (temp, pos);
          w->input_field[IP_YEARS]->value (textbuf->selection_text ());
          textbuf->unselect ();
        }
    }

  /* Find the two spaces before the author's name. */
  if (textbuf->search_forward (0, "  ", &pos) == 1)
    {
      /* Move ahead two positions so we are just in front of the author's
         name. */
      pos += 2;
      temp = pos;

      /* Now look for the " <" string just after the author's name and replace
         it. */
      if (textbuf->search_forward (temp, " <", &pos) == 1)
        {
          textbuf->select (temp, pos);
          w->input_field[IP_AUTHOR]->value (textbuf->selection_text ());
          textbuf->unselect ();
        }
    }

  /* Look for the " <" string just after the author's name. */
  if (textbuf->search_forward (0, " <", &pos) == 1)
    {
      /* Move ahead two positions so we are just in front of the e-mail
         address. */
      pos += 2;
      temp = pos;

      /* Look for the ">" at the end of the e-mail address. */
      if (textbuf->search_forward (temp, ">", &pos) == 1)
        {
          textbuf->select (temp, pos);
          w->input_field[IP_EMAIL]->value (textbuf->selection_text ());
          textbuf->unselect ();
        }
    }

  /* Look for "# Basic". */
  if (textbuf->search_forward (0, "# Basic", &pos) == 1)
    {
      /* Move down one line and go forward two characters. */
      pos = textbuf->skip_lines (pos, 1);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_PACKAGE]->value ());

      /* Move down one line and go forward two characters. */
      pos = textbuf->skip_lines (pos, 1);
      pos = textbuf->line_start (pos);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_VERSION]->value ());

      /* Move down two lines and go forward two characters. */
      pos = textbuf->skip_lines (pos, 2);
      pos = textbuf->line_start (pos);
      pos += 2;

      textbuf->replace (pos, textbuf->line_end (pos),
                        w->input_field[IP_BUILD]->value ());
    }
}

void
set_title (Fl_Window* w)
{
  /* Always start with the name of the program and the version. */
  title.assign ("Dragbuilder 0.1");

  /* If we have a file open display it's name. */
  if (filename[0] != '\0')
    {
      title.append (" - ");
      title.append (filename);
    }

  /* If the file has been modified display a little something. */
  if (changed)
    {
      title.append (" (modified)");
    }

  w->label (title.c_str ());
}

void
changed_cb(int, int nInserted, int nDeleted,int, const char*, void* v)
{
  if ((nInserted || nDeleted) && !loading) changed = 1;
  EditorWindow *w = (EditorWindow *)v;
  set_title (w);
  if (loading) w->display->show_insert_position ();
}

void
new_cb (Fl_Widget*, void*)
{
  if (!check_save())
    {
      return;
    }

  filename[0] = '\0';
  textbuf->select (0, textbuf->length ());
  textbuf->remove_selection ();
  textbuf->loadfile ("template");
  changed = 0;
  textbuf->call_modify_callbacks ();
}

void
open_cb (Fl_Widget*, void*)
{
  if (!check_save ())
    return;

  Fl_Native_File_Chooser fnfc;
  fnfc.title("Open file");
  fnfc.type(Fl_Native_File_Chooser::BROWSE_FILE);
  if ( fnfc.show() ) return;
  load_file(fnfc.filename(), -1);

}

int num_windows = 0;

void
close_cb(Fl_Widget*, void* v)
{
  EditorWindow* w = (EditorWindow*)v;

  if (num_windows == 1)
    {
      if (!check_save ())
          return;
    }

  w->hide ();
  w->display->buffer (0);
  textbuf->remove_modify_callback (changed_cb, w);
  Fl::delete_widget (w);

  num_windows--;
  if (!num_windows) exit (0);
}

void
quit_cb (Fl_Widget*, void*)
{
  if (changed && !check_save ())
    return;

  exit (0);
}

void
save_cb ()
{
  if (filename[0] == '\0')
    {
      /* No filename, get one! */
      saveas_cb ();
      return;
    }
  else
    {
      save_file (filename);
    }
}

void
saveas_cb ()
{
  Fl_Native_File_Chooser fnfc;
  fnfc.title("Save File As?");
  fnfc.type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
  if ( fnfc.show() ) return;
  save_file(fnfc.filename());
}

Fl_Window* new_view();

void
term_show_cb(Fl_Widget*, void* v)
{
  EditorWindow* w = (EditorWindow*)v;

 
}

void
term_hide_cb(Fl_Widget*, void* v)
{
  EditorWindow* w = (EditorWindow*)v;
}

Fl_Menu_Item menuitems[] =
{
  {"&File",       0, 0, 0, FL_SUBMENU},
  {"&New",        FL_COMMAND + 'n', (Fl_Callback *)new_cb},
  {"&Open",       FL_COMMAND + 'o',
   (Fl_Callback *)open_cb, 0, FL_MENU_DIVIDER},
  { "&Save",       FL_COMMAND + 's', (Fl_Callback *)save_cb },
  { "Save &As",   FL_COMMAND + FL_SHIFT + 's', (Fl_Callback *)saveas_cb, 0, FL_MENU_DIVIDER },
  { "Quit",            FL_COMMAND + 'q', (Fl_Callback *)quit_cb, 0 },
  { 0 },

  {"Actions", 0, 0, 0, FL_SUBMENU},
  {"Apply options", FL_ALT + 'a', (Fl_Callback*)apply_cb},
  {"Extract options", FL_ALT + 'e', (Fl_Callback*)extract_cb},
  {"Build package", FL_COMMAND + 'b', 0},
  {0},

  {"Tools", 0, 0, 0, FL_SUBMENU},
  {"Terminal", 0, 0, 0, FL_SUBMENU},
  {"Show",  0, (Fl_Callback*)term_show_cb},
  {"Hide",  0, (Fl_Callback*)term_hide_cb},
  {"Clear", 0, 0},
  {0},
  {0},

  {"Help", 0, 0, 0, FL_SUBMENU},
  {"About", 0, (Fl_Callback*)about_cb},
  {0},

  {0}
};

Fl_Window* new_view()
{
  EditorWindow* w = new EditorWindow(600, 400, title);
  w->begin();
  Fl_Menu_Bar* m = new Fl_Menu_Bar(0, 0, 600, 25);
  m->copy(menuitems, w);
  w->tabs = new Fl_Tabs (0, 25, 600, 375);
  {
    w->gr_script = new Fl_Group (0, 25, 600, 350, "Script");
    {
      w->display = new Fl_Text_Display (0, 25, 600, 355);
      w->display->buffer (textbuf);
      w->display->textfont (FL_COURIER);
      w->display->textsize (14);
      textbuf->text("Welcome to Dragbuilder 0.1!\n\nCreate a new script with the default template with File->New (Ctrl-N) or\nopen a script with File->Open (Ctrl+O).\n\nYou can edit the script by changing tabs below and edit the fields.\n\nTo build a package using the script goto Actions->Build package (Ctrl+B).\n\nSend issues and questions to dragora-bug@nongnu.org.");
    }
    w->gr_script->end();
    w->gr_options = new Fl_Group (0, 30, 600, 350, "Options");
    {
      w->input_field[IP_AUTHOR] = new Fl_Input (LEFT_PAD, 30,
                                                180, 25, "Author:");
      w->input_field[IP_AUTHOR]->value ("Test Nick");

      w->input_field[IP_EMAIL] = new Fl_Input (LEFT_PAD, 60,
                                               180, 25, "E-mail:");
      w->input_field[IP_EMAIL]->value ("frusen@gungre.ch");

      w->input_field[IP_YEARS] = new Fl_Input (LEFT_PAD, 90,
                                               180, 25, "Years:");
      w->input_field[IP_YEARS]->value ("2013-2014");

      w->input_field[IP_PACKAGE] = new Fl_Input (LEFT_PAD, 150,
                                                 180, 25, "Package:");
      w->input_field[IP_PACKAGE]->value ("foo");

      w->input_field[IP_VERSION] = new Fl_Input (LEFT_PAD, 180,
                                                 180, 25, "Version:");
      w->input_field[IP_VERSION]->value ("1.3.2");

      w->input_field[IP_BUILD] = new Fl_Input (LEFT_PAD, 210,
                                               180, 25, "Build:");
      w->input_field[IP_BUILD]->value ("2");
    }
    w->gr_options->end();
  }
  w->tabs->end();
  w->end();
  w->resizable(w->tabs);
  w->callback((Fl_Callback *)close_cb, w);

  textbuf->add_modify_callback(changed_cb, w);
  textbuf->call_modify_callbacks();
  num_windows++;
  return w;
}

void
cb (const char *fname)
{
  load_file(fname, -1);
}

int
main (int argc, char **argv)
{
  textbuf = new Fl_Text_Buffer;
  fl_open_callback (cb);

  Fl::visual (FL_DOUBLE|FL_INDEX);

  Fl_Window* window = new_view ();

  Fl_Double_Window term (320, 240, "Terminal");
  MyTerminal edit(0, 0, term.w(), term.h());
  edit.append ("> ");
  term.resizable (edit);

  window->show (1, argv);

  return Fl::run ();
}
