/*
 * Copyright (C) 2013  Lucas Skldqvist <frusen@gungre.ch>
 *
 * This file is part of Dragbuilder. 
 *
 * Dragbuilder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Dragbuilder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Dragbuilder.  If not, see <http://www.gnu.org/licenses/>. 
 */

#ifndef TERMINAL_H
#define TERMINAL_H

#include <string.h>
#include <stdio.h>
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Editor.H>

class MyTerminal : public Fl_Text_Editor
{
  Fl_Text_Buffer *buff;
  char cmd[1024];
public:
  MyTerminal (int X, int Y,
              int W,int H,
              const char *L=0)
    : Fl_Text_Editor(X,Y,W,H,L)
    {
      buff = new Fl_Text_Buffer();
      buffer(buff);
      textfont(FL_COURIER);
      textsize(12);
      cmd[0] = 0;
    }

  // Append to buffer, keep cursor at end
  void append (const char *s)
  {
    buff->append (s);
    // Go to end of line
    insert_position(buffer()->length());
    scroll(count_lines(0, buffer()->length(), 1), 0);
  }
    // Run the specified command in the shell, append output to terminal
    void RunCommand(const char *command) {
        append("\n");
        fprintf(stderr, "EXECUTING: '%s'\n", command);
        FILE *fp = popen(command, "r");
        if ( fp == 0 ) {
            append("Failed to execute: '");
            append(command);
            append("'\n");
        } else {
            char s[1024];
            while ( fgets(s, sizeof(s)-1, fp) ) {
                append(s);
            }
            pclose(fp);
        }
    }
    // Handle events in the Fl_Text_Editor
    int handle(int e) {
        switch (e) {
            case FL_KEYUP: {
                int key = Fl::event_key();

                /* Disable the enter key.  */
                if (key == FL_Enter)
                  {
                    return 1;
                  }

                /* Do not remove more than the command. */
                if (key == FL_BackSpace && cmd[0] == 0)
                  {
                    return 0;
                  }
                break;
            }
            case FL_KEYDOWN: {
                int key = Fl::event_key();
                // Enter key? Execute the command, clear command buffer
                if ( key == FL_Enter ) {
                    // Execute your commands here
                    strcat(cmd, " 2>&1");                // stderr + stdout
                    RunCommand(cmd);
                    cmd[0] = 0;
                    append("> ");
                    return(1);                          // hide 'Enter' from text widget
                }
                if ( key == FL_BackSpace ) {
                    // Remove a character from end of command buffer
                    if ( cmd[0] ) {
                        cmd[strlen(cmd)-1] = 0;
                        break;
                    } else {
                        return(0);
                    }
                } else {
                    // Append text to our 'command' buffer
                    strncat(cmd, Fl::event_text(), sizeof(cmd)-1);
                    cmd[sizeof(cmd)-1] = 0;
                }
                break;
            }
        }
        return(Fl_Text_Editor::handle(e));
    }
};

#endif /* TERMINAL_H */
